/*
 * Licensed Materials - Property of IBM
 *
 * com.ibm.etools.c
 *
 *  Copyright IBM Corporation 2004. All Rights Reserved. 
 *
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.etools.c.importer;

/**
 * The CTypedElementParser class parses a typed element which has two
 * attributes: name and type. It is used to separate the name and type of a
 * function parameter. The main job is done in the parse() method.
 */
public class CTypedElementParser {

    public static final String copyright = "Licensed Material - Property of IBM\n com.ibm.etools.c\n(C) Copyright IBM Corp. 2004 - All Rights Reserved.\nUS Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp."; //$NON-NLS-1$

    String _signature;

    String _name;

    String _type;

    /**
     * Create a typed element parser from a signature string
     */
    public CTypedElementParser(String signature) {
        _signature = signature.trim();
        parse();
        typeFixup();
    }

    /**
     * Fix up the type by removing extra white spaces
     */
    private void typeFixup() {
        _type = CUtility.normalizedType(_type);
        // Remove the leading const specifier if it exists.
        if (_type != null && _type.startsWith("const "))
            _type = _type.substring(6).trim();
    }

    /**
     * Extract the name and type component from the full signature
     */
    private void parse() {
        // Is it a function pointer?
        if (_signature.indexOf(CIM.P_FUNCTION_PTR) >= 0) {
            int lastBrace = _signature.lastIndexOf(')');
            int lastSpace = _signature.lastIndexOf(' ');
            if (lastSpace > lastBrace) {
                _type = _signature.substring(0, lastBrace + 1);
                _name = _signature.substring(lastSpace + 1).trim();
            } else {
                _name = "";
                _type = _signature;
            }
            return;
        }

        int indexOfFirstBracket = _signature.indexOf(CIM.P_CHAR_LEFT_BRACKET);
        String arraySuffix = new String("");
        if (indexOfFirstBracket > 0) {
            arraySuffix = _signature.substring(indexOfFirstBracket);
            _signature = _signature.substring(0, indexOfFirstBracket).trim();
        }

        int indexOfLastStar = _signature.lastIndexOf(CIM.P_CHAR_POINTER);
        if (indexOfLastStar > 0) {
            _type = _signature.substring(0, indexOfLastStar + 1).trim() + arraySuffix;
            _name = _signature.substring(indexOfLastStar + 1).trim();
            return;
        }

        int indexOfFirstSpace = _signature.indexOf(CIM.P_CHAR_SPACE);
        if (indexOfFirstSpace == -1) {
            _type = _signature + arraySuffix;
            _name = "";
        } else if (_signature.startsWith("struct ") || _signature.startsWith("union ") || _signature.startsWith("enum ")) {
            int indexOfLastSpace = _signature.lastIndexOf(' ');
            if (_signature.substring(indexOfFirstSpace, indexOfLastSpace).trim().length() == 0) {
                _type = _signature + arraySuffix;
                _name = "";
            } else {
                _type = _signature.substring(0, indexOfLastSpace).trim() + arraySuffix;
                _name = _signature.substring(indexOfLastSpace + 1);
            }
        } else {
            _type = _signature.substring(0, indexOfFirstSpace) + arraySuffix;
            _name = _signature.substring(indexOfFirstSpace + 1).trim();
        }
    }

    /**
     * Return the name of the typed element
     */
    public String getName() {
        return _name;
    }

    /**
     * Return the type string of the typed element
     */
    public String getType() {
        return _type;
    }

    /**
     * Standalone Command line driver for testing
     */
    public static void main(String[] args) {

        if (args.length != 1) {
            System.out.println("Usage: java CTypedElement str");
            System.exit(-1);
        }

        CTypedElementParser element = new CTypedElementParser(args[0]);
        System.out.println("Type: " + element.getType());
        System.out.println("Name: " + element.getName());
    }
}